<?php
/**
 * Copyright 2018 Kacper Donat
 *
 * @author Kacper "Kadet" Donat <kacper@kadet.net>
 *
 * Full license available in separate LICENSE file
 */

namespace Kadet\Functional\Utils;

use const Kadet\Functional\_ as PLACEHOLDER;

class PartiallyAppliedFunction
{
    protected $callable;
    protected $arguments = [];

    public function __construct(callable $callable, ...$args)
    {
        switch (true) {
            case $callable instanceof PartiallyAppliedFunction:
                $this->callable  = $callable->callable;
                $this->arguments = $callable->prepareArguments($args);
                break;

            default:
                $this->callable  = $callable;
                $this->arguments = array_values($args);
        }
    }

    public function __invoke(...$args)
    {
        $arguments = $this->prepareArguments($args);
        return ($this->callable)(...$arguments);
    }

    private function prepareArguments($args)
    {
        $result = [];
        foreach ($this->arguments as $argument) {
            $result[] = $argument === PLACEHOLDER
                ? array_shift($args)
                : $argument;
        }

        return array_merge(
            array_values($result),
            array_values($args)
        );
    }
}